"""
	Smart Multifaceted Platform ™
	© 2019 - 2024 - Bouygues Energies & Services

	Bibliothèque de gestion VIDEO
"""

import system
import shared.commun.constantes
import shared.commun.constantesCustom
import shared.commun.logger

import shared.commun.video
import system.byes.webrtc

import perspective.gui.utils

strLib = "perspective.gui.video"

def openPopup(session,key,output_codec="",large=False,positionWithoutWH={},quality=""):
	"""
		output_codec : h264 ou vp8 ou vp9
	"""
	global strLib
	strFct="openPopup"
	try:
		view = "popups/STD/STD_POPUP_VIDEO"
		title = u"vidéo : %s" % key

		# pas valable si le navigateur n'est pas en fullscreen
		# utiliser des dimmension fixe
		if large:
			width = 88
			height = 88
			position = {'width':'1280px','height':'720px'}
		else:
			width = 60
			height = 60
			position = {'width':'1024px','height':'576px'}
		#position = {'width':'%d%%' % width,'height':'%d%%' % height}	
		
		if not positionWithoutWH == {}:
			if 'right' in positionWithoutWH:
				position['right'] = positionWithoutWH['right']
			if 'left' in positionWithoutWH:
				position['left'] = positionWithoutWH['left']
			if 'top' in positionWithoutWH:
				position['top'] = positionWithoutWH['top']
			if 'bottom' in positionWithoutWH:
				position['bottom'] = positionWithoutWH['bottom']	
			
		params = {}		
		params["key"] = key
		params["large"] = large
		params["latency"] = 0
		params["debug"] = True
		params["autoplay"] = True

		dicoCamera = {}
		dicoCamera["endpointAddress"] = ""
		dicoCamera["endpointUser"] = ""
		dicoCamera["endpointPassword"] = ""			
		dicoCamera["ptzActivePanTilt"] = False
		dicoCamera["ptzActiveZoom"] = False
		dicoCamera["ptzProfileToken"] = False
		dicoCamera["streamUser"] = ""
		dicoCamera["streamPassword"] = ""
		dicoCamera["streamUrl"] = ""
		dicoCamera["eqtPath"] = ""	
		
		if key.startswith("pattern") or key == "":
			# cas des patterns gstreamer de test
			dicoCamera["streamUrl"] = key
			if output_codec != "":
				params["output_codec"] = output_codec
				params["quality"] = quality
			else:
				params["output_codec"] = "h264"
				params["quality"] = ""
		else:	
			# recherche des données de configuration de la caméra
			dicoCamera = shared.commun.video.rechercherCamera(key)
			
			# récupération des settings pour affichage vidéo configurés lors de initModuleKeys()
			if output_codec == "":
				dico = system.byes.webrtc.getAllStreamsConfig()
				if dico != None:
					if key in dico:
						params["output_codec"] = dico[key]["outputCodec"]
						params["quality"] = dico[key]["outputCodecQuality"]
					else:
						params["output_codec"] = "h264"
						params["quality"] = ""
			else:
				params["output_codec"] = output_codec
				params["quality"] = quality	
		
		if (dicoCamera != None and len(dicoCamera)>0):
			params["endpointAddress"] = dicoCamera["endpointAddress"]
			params["endpointUser"] = dicoCamera["endpointUser"]
			params["endpointPassword"] = dicoCamera["endpointPassword"]			
			params["ptzActivePanTilt"] = dicoCamera["ptzActivePanTilt"]
			params["ptzActiveZoom"] = dicoCamera["ptzActiveZoom"]
			params["ptzProfileToken"] = dicoCamera["ptzProfileToken"]
			params["streamUser"] = dicoCamera["streamUser"]
			params["streamPassword"] = dicoCamera["streamPassword"]
			params["streamUrl"] = dicoCamera["streamUrl"]
			#params["idCam"] = dicoCamera["idCam"]
			params["eqtPath"] = dicoCamera["eqtPath"]
	
		system.perspective.closePopup("popup-video")
		system.perspective.openPopup("popup-video",view,params,title=title,position=position,showCloseIcon=True,resizable=False,modal=False,overlayDismiss=False)
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)

def affecterIncrustationVideoMultiple(session,position,tagCamera):
	global strLib
	strFct="affecterIncrustationVideoMultiple"	
	try:			
		shared.commun.logger.logDEBUG(strFct=strFct,strLib=strLib,strMsg="position=%s,tagCamera=%s"%(position,tagCamera))
		
		if tagCamera.startswith("pattern") or tagCamera == "":		
			# cas des patterns gstreamer de test ou de la RAZ de l'affectation
			dicoCamera = {}
			dicoCamera["endpointAddress"] = ""
			dicoCamera["endpointUser"] = ""
			dicoCamera["endpointPassword"] = ""			
			dicoCamera["ptzActivePanTilt"] = False
			dicoCamera["ptzActiveZoom"] = False
			dicoCamera["ptzProfileToken"] = False
			dicoCamera["streamUser"] = ""
			dicoCamera["streamPassword"] = ""
			dicoCamera["streamUrl"] = tagCamera
			dicoCamera["eqtPath"] = ""					 
		else:	
			# recherche des données de configuration de la caméra
			resultDicoCamera = shared.commun.video.rechercherCamera(tagCamera)
			if (resultDicoCamera != None and len(resultDicoCamera)>0):
				dicoCamera = resultDicoCamera
		
		# envoie message pour vue du mur d'image local
		payload={}
		payload["position"] = position
		#payload["eqtPath"] = eqtPath
		payload["tagCamera"] = tagCamera				
		
		payload["endpointAddress"] = dicoCamera["endpointAddress"]
		payload["endpointUser"] = dicoCamera["endpointUser"]
		payload["endpointPassword"] = dicoCamera["endpointPassword"]			
		payload["ptzActivePanTilt"] = dicoCamera["ptzActivePanTilt"]
		payload["ptzActiveZoom"] = dicoCamera["ptzActiveZoom"]
		payload["ptzProfileToken"] = dicoCamera["ptzProfileToken"]
		payload["streamUser"] = dicoCamera["streamUser"]
		payload["streamPassword"] = dicoCamera["streamPassword"]
		payload["streamUrl"] = dicoCamera["streamUrl"]
		#payload["idCam"] = dicoCamera["idCam"]
		payload["eqtPath"] = dicoCamera["eqtPath"]
		
		# Message pour le client vision qui tourne sur le même PC
		project = shared.commun.constantesCustom.PEER_PROJECT_VIDEO_WALL

		messageHandler = "IHM_AffecterIncrustationVideoMultiple"
		scope = "C"
		#hostName = session.props.host
		
		payload["ipAddress"] = perspective.gui.utils.getClientIpAddress(session)
		
		strMsg="sendMessage : project=%s, messageHandler=%s, scope=%s, payload=%s" % (project,messageHandler,scope,payload)
		shared.commun.logger.logDEBUG(strFct=strFct,strLib=strLib,strMsg=strMsg)
		
		# session.props.host pour filtrage hostName contient une @IP au lieu d'un host
		# sendMessage avec filtre hostName ne fonctionne pas			 
		#,hostName=hostName)
		# Gestion filtre manuel avec [System]Client/Network/IPAddress
		# et cas particulier localhsot/127.0.0.1
		
		system.util.sendMessage(project=project,messageHandler=messageHandler,payload=payload,scope=scope)
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)

def initModuleKeys():
	"""
		initialise les clés utilisées par le module BYES-VideoViewerNextGen
		pour stocker les paramètres des flux rtsp et les échanger avec l'iFrame
	"""
	global strLib
	strFct="initModuleKeys"
	try:
		system.byes.webrtc.removeAllStreamsConfig()
		
		dicoConfig = shared.commun.constantes.getAllConstantesVIDEO()
		
		demoMode = False
		if "demoMode" in dicoConfig:
			demoMode = dicoConfig["demoMode"]
		if demoMode:
			# mode de test avec une ou plusieurs caméra
			demoCamera = dicoConfig["demoCamera"]
			for key in demoCamera.keys():			
				rtspUrl = demoCamera[key]["streamUrl"]
				user = demoCamera[key]["streamUser"]
				password = demoCamera[key]["streamPassword"]
				#rtspPipelineProperties = "onvif-mode=true"
				#system.byes.webrtc.addStreamConfig(key,rtspUrl,user,password,rtspPipelineProperties=rtspPipelineProperties)			
				if "output_stream_process" in demoCamera[key]:
					output_stream_process = demoCamera[key]["output_stream_process"]
				else:
					output_stream_process = ""
				if "pipeline" in demoCamera[key]:
					pipeline = demoCamera[key]["pipeline"]
				else:
					pipeline = ""
				
				if (pipeline != ""):
					rtspUrl = ""
					system.byes.webrtc.addStreamConfig(key,rtspUrl,pipeline=pipeline)
				else:
					(outputCodec,outputCodecQuality,decodeEncode) = getQualityStreamProcess(output_stream_process)
					rtph264payOption = shared.commun.constantesCustom.VIDEO_SERVER.get("rtph264payOption",{}).get(output_stream_process,"")
					system.byes.webrtc.addStreamConfig(key,rtspUrl,user,password,
						decodeEncode=decodeEncode,
						outputCodec=outputCodec,
						outputCodecQuality=outputCodecQuality,
						rtph264payOption=rtph264payOption)	
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)

def getRemoteAllStreamsConfig(ignitionSystem,project):
	"""
		Interroge la gateway dédié au décodage vidéo afin d'obtenir les streams configurés
		avec system.byes.webrtc.addStreamConfig
	"""
	global strLib
	strFct="getRemoteAllStreamsConfig"
	try:
		messageHandler = "gwHandlerGetAllStreamsConfig"
		remoteServer = ignitionSystem		
		shared.commun.logger.logINFO(strMsg="remoteServer = %s, project = %s messageHandler = %s"
			%(remoteServer, project,messageHandler),strFct=strFct,strLib=strLib)
		
		results = system.util.sendRequest(project,messageHandler,remoteServer=remoteServer)
		shared.commun.logger.logINFO(strMsg="results=%s" % results,strFct=strFct,strLib=strLib)
		return results
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)
		return None

def getQualityStreamProcess(output_stream_process):
	"""Détermine le traitement ou non à appliquer sur le flux vidéo pour qu'il soit affiché dans perspective
		
		output_stream_process	rtspsrc     output_codec    decode_encode    pipeline
		--------------------------------------------------------
        h264					h264        h264       		false       	parsebin
        h264_enc				h264        h264       		true        	decodebin + x264enc ou autre encodeur openh264enc
        h264_enc				h265        h264       		-           	decodebin + x264enc ou autre encodeur openh264enc
        vp8 ou vp9				h265        vp8 ou vp9 		-           	decodebin + vp8enc/vp9enc
        
        /!\ h265 : Fonctionne uniquement si chrome est lancé avec les paramètres:
        et si le poste duspose d'une carte NVIDIA pour du décodage matériel. Sinon le navigateur ne supporte pas de décodeur logiciel
        cd "C:\Program Files\Google\Chrome\Application"
		chrome.exe --enable-features=WebRtcAllowH265Receive --force-fieldtrials=WebRTC-Video-H26xPacketBuffer/Enabled
        h265					h265        h265       		false       	parsebin
        
		h264_enc est conservé pour reto-compatibilité
		=> utilise un codec en dur dans le module

		x264enc					h264/h265	x264enc			true			decodebin + "output_stream_process"
		openh264enc							...
		nvh264enc
		nvautogpuh264enc
		nvcudah264enc

        - = paramètre non pris en compte => encodage obligatoire
		
		quality à fournir uniquement si decodebin
		sinon ""
		
		return tuple(output_codec, quality, decode_encode)
		
		carte NVIDIA Quadro P400 / T1000
		gst-inspect-1.0 | findstr nv
		nvcodec:  nvautogpuh264enc: NVENC H.264 Video Encoder Auto GPU select Mode
		nvcodec:  nvautogpuh265enc: NVENC H.265 Video Encoder Auto GPU select Mode
		nvcodec:  nvcudah264enc: NVENC H.264 Video Encoder CUDA Mode
		nvcodec:  nvcudah265enc: NVENC H.265 Video Encoder CUDA Mode
		nvcodec:  nvd3d11h264enc: NVENC H.264 Video Encoder Direct3D11 Mode
		nvcodec:  nvd3d11h265enc: NVENC H.265 Video Encoder Direct3D11 Mode
		nvcodec:  nvh264dec: NVDEC H.264 Decoder
		nvcodec:  nvh264enc: NVENC H.264 Video Encoder
		nvcodec:  nvh265dec: NVDEC H.265 Decoder
		nvcodec:  nvh265enc: NVENC HEVC Video Encoder		
		
		decodebin choisi directement le meilleur décodeur, celui pour la carte nvidia si elle est dispo
		pour le re-encodage en h264
		
	"""	
	try:
		output_codec = "h264"
		quality = ""
		decode_encode = False
		if output_stream_process in ["h264","",None]:
			output_codec = "h264"
			quality = ""
			decode_encode = False
		elif output_stream_process == "h264_enc":
			output_codec = "h264"
			quality = ""
			if "outputStreamProcessQuality" in shared.commun.constantesCustom.VIDEO_SERVER:
				outputStreamProcessQuality = shared.commun.constantesCustom.VIDEO_SERVER.get("outputStreamProcessQuality")
				quality = outputStreamProcessQuality.get(output_stream_process,"")			
			decode_encode = True			
		elif output_stream_process in ["x264enc","openh264enc","nvh264enc","nvautogpuh264enc","nvcudah264enc"]:
			output_codec = output_stream_process
			quality = ""
			if "outputStreamProcessQuality" in shared.commun.constantesCustom.VIDEO_SERVER:
				outputStreamProcessQuality = shared.commun.constantesCustom.VIDEO_SERVER.get("outputStreamProcessQuality")
				quality = outputStreamProcessQuality.get(output_stream_process,"")			
			decode_encode = True
		elif output_stream_process in ["vp8","vp9"]:
			output_codec = output_stream_process
			quality = ""
			if "outputStreamProcessQuality" in shared.commun.constantesCustom.VIDEO_SERVER:
				outputStreamProcessQuality = shared.commun.constantesCustom.VIDEO_SERVER.get("outputStreamProcessQuality")
				quality = outputStreamProcessQuality.get(output_stream_process,"")
			else:
				shared.commun.logger.logERROR("outputStreamProcessQuality key must be provided in shared.commun.constantesCustom.VIDEO_SERVER with output_stream_process = %s" % output_stream_process)			
			#quality = "deadline=1 target-bitrate=8000000 threads=4"
			# pas utile de le renseigne pour le module video, re-encodage obligatoire pour vp8,vp9
			decode_encode = False
		elif output_stream_process in ["h265"]:
			output_codec = output_stream_process
			quality = ""
			outputStreamProcessQuality = shared.commun.constantesCustom.VIDEO_SERVER.get("outputStreamProcessQuality")
			quality = outputStreamProcessQuality.get(output_stream_process,"")			
		else:
			shared.commun.logger.logERROR("output_stream_process = %s not supported (expected values : h264,h264_enc,vp8,vp9)" % stream_process)
	except:
		shared.commun.logger.logERROR()
	return (output_codec, quality,decode_encode)
