"""
	Smart Multifaceted Platform ™
	© 2019 - 2022 - Bouygues Energies & Services

	Bibliothèque utilisées pour le Joystick video
"""

import system
import shared.commun.logger
import shared.commun.constantes

if shared.commun.constantes.getConstantePLATFORM("video_on"):
	import system.byes.onvif.direct
	
if shared.commun.constantes.getConstantePLATFORM("joystick_on"):	
	import system.byes.joystick

#https://forum.inductiveautomation.com/t/client-scope-notify-script/21499/7
#It is an artifact of backward compatibility to some old versions of Ignition.
#Only on some event functions. Functions and classes defined within those events have a shared globals namespace and an empty local namespace.
#It is recommended that all real functions and classes that you will be passing around as objects be defined in a shared.* or project.* script module,
#where modern per-script-module scoping rules apply. Such event routines basically just call the script module function and pass any event object to it.
#That avoids this whole mess.

strLib = "project.gui.joystick"

ptzActiveZoom = True
ptzActivePanTilt = True

# Mémo de la réservation du joystick
reserveProfileToken = ""
# incrust ou mur
reserveSource = ""
listPreset = []

# Joystick Axis AXIS T831
# name=Axe X, identifierName=x
# name=Bouton 0, identifierName=0

# pour mémo en var global afin de ne pas relire le tag client
# ou si trop rapide pour tag client
axesValue = {
	"x" : 0,
	"y" : 0,
	"z" : 0
}

previousAxesValue = {
	"x" : 0,
	"y" : 0,
	"z" : 0
}

buttonsValue = {
	"0" : False,
	"1" : False,
	"2" : False,
	"3" : False,
	"4" : False,
	"5" : False			
}

tagPathAxes = {
	"x" : "[Client]_intern/joystick/axe_x",
	"y" : "[Client]_intern/joystick/axe_y",
	"z" : "[Client]_intern/joystick/axe_z"
}

tagPathButtons = {
	"0" : "[Client]_intern/joystick/button_0",
	"1" : "[Client]_intern/joystick/button_1",
	"2" : "[Client]_intern/joystick/button_2",
	"3" : "[Client]_intern/joystick/button_3",
	"4" : "[Client]_intern/joystick/button_4",
	"5" : "[Client]_intern/joystick/button_5"			
}

tagPathIsConnected = "[Client]_intern/joystick/isConnected"
tagPathPanTiltEnd = "[Client]_intern/joystick/panTiltEnd"
tagPathZoomEnd = "[Client]_intern/joystick/zoomEnd"
tagJoystickPostes = "[Client]_intern/joystick/postes"

def isJoystickExpected():
	global strLib
	strFct="isJoystickExpected"
	
	global tagJoystickPostes
	
	try:	
		data = system.tag.read(tagJoystickPostes).value
		ipClient = system.tag.read("[System]Client/Network/IPAddress").value
		rowsToDelete = []
		for row in range(data.getRowCount()):
			ip = data.getValueAt(row,"IPClientGTC")
			if ((ip == ipClient) or (ip == "*")):
				return True
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)
	return False

# axe string identifierName
def setAxeValue(axeId,value):
	global strLib
	strFct="setAxeValue"	
	global tagPathAxes
	try:
		if axeId in axesValue.keys():
			axesValue[axeId]=value
			system.tag.write(tagPathAxes[axeId],value)
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)

def getAxeValue(axeId):
	global strLib
	strFct="getAxeValue"	
	global tagPathAxes
	try:
		return axesValue[axeId]
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)
		return 0

def setButtonValue(buttonId,value):
	global strLib
	strFct="setButtonValue"	
	global tagPathButtons			
	try:
		if buttonId in buttonsValue.keys(): 
			buttonsValue[buttonId]=value
			system.tag.write(tagPathButtons[buttonId],value)
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)

def getButtonValue(buttonId):
	global strLib
	strFct="getButtonValue"	
	global tagPathButtons			
	try:
		return buttonsValue[buttonId]
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)
		return False

def initAxesAndButtonsValue():
	global strLib
	strFct="initAxesAndButtonsValue"
	global tagPathAxes	
	global tagPathButtons	
	try:
		for key, value in tagPathAxes.iteritems():
			system.tag.write(value,0)
		for key, value in tagPathButtons.iteritems():			
			system.tag.write(value,False)			
		panTiltEnd = True
		zoomEnd = True
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)

#payload avec les keys : num, name, identifierName, pollData, deadZone, isRelative
def callbackAxes(payload):
	global strLib
	strFct="callbackButtons"
	
	global tagPathPanTiltEnd
	global tagPathZoomEnd
		
	global previousAxesValue

	global ptzActiveZoom
	global ptzActivePanTilt
	
	global reserveProfileToken
		
	try:
	
		strMsg = "callbackAxes,payload=%s"%str(payload)
		shared.commun.logger.logTRACE(strMsg=strMsg,strFct=strFct,strLib=strLib,dicoTrace=payload)
		
		#bande_morte = 0.01
		#bande_morte = 0.0001
		bande_morte = 0.05
		
		panTiltEnd_prec = system.tag.read(tagPathPanTiltEnd).value
		zoomEnd_prec = system.tag.read(tagPathZoomEnd).value
		
		if system.byes.onvif.direct.ptzIsPortBinded():
			shared.commun.logger.logTRACE(strFct=strFct,strLib=strLib,dicoTrace=payload)
			num = payload["num"]
			name  = payload["name"]
			identifierName = payload["identifierName"]
			pollData = payload["pollData"]
			deadZone = payload["deadZone"]
			isRelative = payload["isRelative"]
			
			setAxeValue(identifierName,pollData)
			
			# Envoi d'une commande PTZ par Axe X et Y, pas de combinaison
			# A voir ... pour combiner, il faudrait gérer surveiller les valeurs X, Y et Z mémorisées
			
			profileToken = reserveProfileToken
			
#			if (abs(previousAxesValue[identifierName]-pollData) >  bande_morte):

			panTiltEnd = True
			zoomEnd = True
			
			if ((identifierName == "x") and (abs(pollData) > bande_morte)):
				if ptzActivePanTilt:
					system.byes.onvif.direct.ptzContinuousMove(profileToken=profileToken,panTiltX=pollData)
				panTiltEnd = False
			if ((identifierName == "y") and (abs(pollData) > bande_morte)):
				if ptzActivePanTilt:
					system.byes.onvif.direct.ptzContinuousMove(profileToken=profileToken,panTiltY=pollData)
				panTiltEnd = False
			if ((identifierName == "z") and (abs(pollData) > bande_morte)):
				if ptzActiveZoom:
					system.byes.onvif.direct.ptzContinuousMove(profileToken=profileToken,zoom=pollData)
				zoomEnd = False

			# memo valeur
			previousAxesValue[identifierName] = pollData
			system.tag.write(tagPathPanTiltEnd,panTiltEnd)
			system.tag.write(tagPathZoomEnd,zoomEnd)

			# commande de STOP mais pas en permanence !
			if (panTiltEnd == True and panTiltEnd_prec != True):
				# stop panTilt
				if ptzActivePanTilt:
					system.byes.onvif.direct.ptzStop(profileToken=profileToken,panTilt=True,zoom=False)				
			if (zoomEnd == True and zoomEnd_prec != True):
				# stop zoom
				if ptzActiveZoom:				
					system.byes.onvif.direct.ptzStop(profileToken=profileToken,zoom=True,panTilt=False)

	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)

#payload avec les keys : num, name, identifierName, buttonPressed
def callbackButtons(payload):
	global strLib
	strFct="callbackButtons"
	
	global ptzActiveZoom
	global ptzActivePanTilt

	global reserveProfileToken
	
	global listPreset
			
	try:
		if system.byes.onvif.direct.ptzIsPortBinded():
			shared.commun.logger.logTRACE(strFct=strFct,strLib=strLib,dicoTrace=payload)
			num = payload["num"]
			name  = payload["name"]
			identifierName = payload["identifierName"]
			buttonPressed = payload["buttonPressed"]			
			setButtonValue(identifierName,buttonPressed)
			
			if ptzActivePanTilt and buttonPressed:
				#identifierName : 0, 1, 2, 3
				#				  4 (L)    5 (R)
				try:
					identifierName = int(identifierName)
					profileToken = reserveProfileToken
					if ((identifierName in [0,1,2,3]) and (identifierName < len(listPreset))):					
						presetToken = listPreset[identifierName]
						presetToken = presetToken.token
						system.byes.onvif.direct.ptzGotoPreset(profileToken=profileToken,presetToken=presetToken)
					# Button L et R => gotoHome
					if (identifierName in [4,5]):
						system.byes.onvif.direct.ptzGotoHomePosition(profileToken=profileToken)
				except ValueError:
					pass												
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)

def startPollController():
	global strLib
	strFct="startPollController"		
		
	try:
		#print "startPollController"
	
		initAxesAndButtonsValue()
		if not system.byes.joystick.direct.isControllerConnected():
			strMsg="Controller is not connected"
			shared.commun.logger.logDEBUG(strMsg,strFct=strFct,strLib=strLib)
		else:
			system.byes.joystick.direct.registerCallbackButtons(callback=project.gui.joystick.callbackButtons)
			system.byes.joystick.direct.registerCallbackAxes(callback=project.gui.joystick.callbackAxes)	
			
			# showTrace : 0 : aucune, 1 : état initial uniquement, 2 : tous les changements d'état
			showTrace = 2
			
			# pour le filtrage des callback de notification dans le module
			deltaPollDataNotification = 0.01

			buttonOnlyTrueNotification = False
			pollScanMs = 25
			system.byes.joystick.direct.startPollController(showTrace=showTrace,
					deltaPollDataNotification=deltaPollDataNotification,
					buttonOnlyTrueNotification=buttonOnlyTrueNotification,
					pollScanMs=pollScanMs)
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)

def stopPollController():
	global strLib
	strFct="stopPollController"	
	try:
		system.byes.joystick.direct.stopPollController()
		system.byes.joystick.direct.unregisterCallbacks()
		initAxesAndButtonsValue()		
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)

# Connexion/Reconnexion automatique du Joystick
# Fonction à appeler cycliquement
# return True si Joystick connecté
def autoConnectController():
	global strLib
	strFct="autoConnectController"
	
	global listPreset
	global reserveProfileToken
	global reserveSource
	
	global tagPathIsConnected
	
	try:
		isConnected = False
		if not system.byes.joystick.direct.isControllerConnected():
			# True => force hardware scan => permet de retoruver le joystick en cas de deconnexion / reconnexion usb
			controller = system.byes.joystick.direct.searchForController(True)
			
			# supprime la réservation
			reserveProfileToken = ""
			reserveSource = ""
			listPreset = []
			stopPollController()			
			
			if controller != "":
				isConnected = True
			else:
				isConnected =  False
		else:
			isConnected =  True
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)
	system.tag.write(tagPathIsConnected,isConnected)	
	return isConnected

def init():
	try:
		global tagPathIsConnected
		global tagPathPanTiltEnd
		global tagPathZoomEnd
	
		strFct="initJoystick"	
		if shared.commun.constantes.getConstantePLATFORM("joystick_on"):		
			initAxesAndButtonsValue()
			tagPaths = [tagPathIsConnected,
						tagPathPanTiltEnd,
						tagPathZoomEnd]
			values = [False,False,False]					
			system.tag.writeAll(tagPaths,values)
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)

# Nota : utilise le endpoint affecté pour la mire ptz
def reserveController(profileToken,source="incrust"):
	global listPreset
	global reserveProfileToken
	global reserveSource
	strFct="reserveController"
	try:
		if ((profileToken != "") and (system.byes.joystick.direct.isControllerConnected())):
			strMsg="profileToken=%s" % (profileToken)
			shared.commun.logger.logDEBUG(strMsg=strMsg,strFct=strFct,strLib=strLib)
			# Récupération des presets
			if system.byes.onvif.direct.ptzIsPortBinded():
				reserveProfileToken = profileToken
				reserveSource = source
				listPreset = system.byes.onvif.direct.ptzGetPresets(profileToken=profileToken)
				if listPreset == None:
					listPreset = []				
				# on conserve seulement les 4 premières preset
				# pour correspondre aux 4 boutons
				listPreset = listPreset[:4]			
			startPollController()
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)

# controle que cela correspond à la réservation afin de pouvoir appeler la fonction
# sour toutes les actions même si la vue n'a pas réservé le joystick
def freeController(endpointAddress,profileToken,source="incrust"):
	global listPreset
	global reserveProfileToken
	global reserveSource	
	strFct="freeController"
	try:
		if ((profileToken != "") and endpointAddress != ""):
			endpointAddressBinded = system.byes.onvif.direct.ptzGetEndpointAddressBinded()
			if isControllerReserved(endpointAddress,profileToken,source):
				strMsg="endpointAddress=%s - profileToken=%s - source=%s" % (endpointAddress,profileToken,source)
				shared.commun.logger.logDEBUG(strMsg=strMsg,strFct=strFct,strLib=strLib)
				reserveProfileToken = ""
				reserveSource = ""
				listPreset = []
				stopPollController()
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)

# Fonction appelée aussi en runscript pour savoir si la fenêtre vidéo possède le joystick
def isControllerReserved(endpointAddress,profileToken,source="incrust"):
	global reserveProfileToken
	global reserveSource
	try:
		if reserveSource != "":
			endpointAddressBinded = system.byes.onvif.direct.ptzGetEndpointAddressBinded()
			if ((reserveProfileToken == profileToken) and (endpointAddress == endpointAddressBinded) and (reserveSource == source)):
				return True
			else:
				return False
		else:
			return False
	except:
		return False
	
def printControllerComponents():
	if autoConnectController():
		print system.byes.joystick.direct.getControllerComponents()
	else:
		print "Not connected"	



