"""
	Smart Multifaceted Platform ™
	© 2019 - 2022 - Bouygues Energies & Services

	Bibliothèque de gestion des popup de l'application
"""

import system
import shared.commun.logger
import project.gui.constantes

import project.nav.windows

import shared.commun.constantes
video_on = shared.commun.constantes.getConstantePLATFORM("video_on")

strLib = "project.gui.popup"

pathPopupAbout = "STD/POPUP/POPUP_ABOUT"

def errorBox(message,desktop=""):
	global strLib
	strFct="errorBox"
	
	try:
		closeDiagnosticsDialog()
		
		title="Erreur"
		if desktop=="":
			desktop = system.gui.getCurrentDesktop()

		return system.gui.desktop(desktop).errorBox(message,title)
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)
	
def warningBox(message,desktop=""):
	global strLib
	strFct="warningBox"

	try:
		closeDiagnosticsDialog()
	
		title="Avertissement"
		if desktop=="":
			desktop = system.gui.getCurrentDesktop()		
		return system.gui.desktop(desktop).warningBox(formatMessage % message,title)
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)

def messageBox(message,desktop=""):
	global strLib
	
	strFct="messageBox"
	try:
		closeDiagnosticsDialog()
	
		title="Information"
		if desktop=="":
			desktop = system.gui.getCurrentDesktop()
		
		return system.gui.desktop(desktop).messageBox(message,title)
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)

def inputBox(message,defaultText,desktop=""):
	global strLib
	strFct="inputBox"

	try:
		closeDiagnosticsDialog()
	
		if desktop=="":
			desktop = system.gui.getCurrentDesktop()
		return system.gui.desktop(desktop).inputBox(message,defaultText)
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)

def confirm(message,allowCancel=False,desktop=""):
	global strLib
	strFct="confirm"
	try:
		closeDiagnosticsDialog()
	
		title="Confirmation"
		if desktop=="":
			desktop = system.gui.getCurrentDesktop()
					
		return system.gui.desktop(desktop).confirm(message,title,allowCancel)
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)


def openPopup(eventSource,desktop,pathPopup,params):
	global strLib
	strFct="openPopup"
			
	try:
		if desktop=="":
			desktop = system.gui.getCurrentDesktop()	
		
		# Fermeture de tous les popup sauf le popup video
		closePopup(desktop)
		# Popup vidéo rendu invisible
#		project.gui.video.hidePopupVideo(desktop)
		system.nav.desktop(desktop).openWindow(pathPopup,params)
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)


def openFile(event,message,defaultPath,extensionFilter="*.csv",extensionFilters=["csv"]):
	global strLib
	strFct="openFile"
	from javax.swing import JFileChooser
	from javax.swing.filechooser import FileNameExtensionFilter
	
	try:
		chooser = JFileChooser(defaultPath)
		chooser.setFileSelectionMode(JFileChooser.FILES_ONLY)
		chooser.setAcceptAllFileFilterUsed(False)
		
		if extensionFilter != "" and extensionFilters != []:
			filter = FileNameExtensionFilter(extensionFilter,extensionFilters)
			chooser.setFileFilter(filter)
		
		chooser.setDialogTitle(message)
		window = system.gui.getParentWindow(event)	
		
		if chooser.showOpenDialog(window) == JFileChooser.APPROVE_OPTION:
			return chooser.getSelectedFile().getPath()
		else:
			return None
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)
		return None

def openDirectory(event,message,defaultPath):
	global strLib
	strFct="openDirectory"
	from javax.swing import JFileChooser
	
	try:
		chooser = JFileChooser(defaultPath)
		chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY)	
		chooser.setAcceptAllFileFilterUsed(False)
		chooser.setDialogTitle(message)
		window = system.gui.getParentWindow(event)
		if chooser.showOpenDialog(window) == JFileChooser.APPROVE_OPTION:
			return chooser.getSelectedFile().getPath()
		else:
			return None
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)
		return None

def optionBox(message,options=[],defaultOption="",desktop=""):
#	import project.nav.windows
	
	global strLib
	strFct="optionBox"
	from javax.swing import JOptionPane

	global formatMessage

	try:
		if desktop=="":
			desktop = system.gui.getCurrentDesktop()
		currentWindow = project.nav.windows.getCurrentWindow(desktop)
		jop = JOptionPane()
		title = u'Options'
		messageType = JOptionPane.QUESTION_MESSAGE
		icon = None
		
		optionsHtml = []
		for option in options:
			optionsHtml.append("<html><b>%s</b></html>" % option)		
		cr = jop.showInputDialog(currentWindow,formatMessage % message,title,messageType,icon,optionsHtml,defaultOption)
		cr = cr.replace("<html><b>","")
		cr = cr.replace("</b></html>","")		
		return cr
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)
		return None

def openPopupAbout(desktop=""):
	global strLib
	strFct="openPopupAbout"
	
	global pathPopupAbout
		
	try:
		if desktop=="":
			desktop = system.gui.getCurrentDesktop()
		window = system.nav.desktop(desktop).openWindow(pathPopupAbout,{})
		system.nav.desktop(desktop).centerWindow(pathPopupAbout)
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)


from javax.swing import SwingUtilities
from com.inductiveautomation.factorypmi.application import FPMIApp
from org.apache.commons.lang3.reflect import FieldUtils
from javax.swing import JFrame

frameDiagnosticsDialog = None

def closeDiagnosticsDialog():
	try:
		global frameDiagnosticsDialog
		if frameDiagnosticsDialog != None:
			frameDiagnosticsDialog.setVisible(False)
		else:
			fpmiapp = FPMIApp.getInstance()
			if fpmiapp is not None:
				obj = FieldUtils.readField(fpmiapp,"diagnosticsDialog",True)
				frameDiagnosticsDialog = SwingUtilities.getAncestorOfClass(JFrame,obj)
				frameDiagnosticsDialog.setVisible(False)
	except:
		pass
