"""
	Smart Multifaceted Platform ™
	© 2019 - 2022 - Bouygues Energies & Services

	Bibliotheque de gestion du logger de l'application
		strNomLogger = Nom du logger de l'application
		LEVEL gere (ERROR / WARNING / INFO / DEBUG / TRACE)
"""

import system
import shared.commun.constantesCustom

strLib = "shared.commun.logger"

# NB si le lecteur n'est pas dispo sur le client => utilise automatiquement le c:
logDir = "e:/log"
logMaxFileSize = "1 MB"
logMaxIndex = 10

logPattern = "%date{yyyy/MM/dd HH:mm:ss.SSS} [%thread] %-5level %logger{35} - %msg%n"
appenderName = "byes-vision"

rootLoggerByesModules = "com.bouyguesenergiesservices"


#Definition des niveaux de log par défaut
#strCDefaultlogLevel = "DEBUG"
#strGDefaultLogLevel = "DEBUG"

#logger a declarer
strDefaultNomLogger = shared.commun.constantesCustom.ROOT_LOGGER

mdcScriptKey = "byes-script"
# la valeur correspond au nom du logger
# http://files.inductiveautomation.com/sdk/javadoc/ignition79/795/index.html
# https://forum.inductiveautomation.com/t/add-context-key-to-a-logger/26291/3

mdcSessionKey = "session-id"

#--------------------------------------------------------------------------------------------------------------

def init():
	"""
		Fonction d'initialisation du dictionnaire des logger 
		(dicoLogger)
		ou mise à jour des niveaux des loggers existant
	"""
	global dicoLogger
	#global lstNomLogger
	global strDefaultNomLogger
	
	#Niveau de log à partir de celui mémorisé dans Tag Ignition
	level = getLogLevel()
	
	#Enregistrement du Logger principal
	if strDefaultNomLogger not in dicoLogger: 
		dicoLogger[strDefaultNomLogger] = system.util.getLogger(strDefaultNomLogger)
		system.util.setLoggingLevel(strDefaultNomLogger, level)
	
	for loggerName in dicoLogger:
		system.util.setLoggingLevel(loggerName, level)		

def initModules():
	"""
		Fonction d'initialisation des niveaux des logger des modules byes 	
	"""
	global rootLoggerByesModules
	#Niveau de log à partir de celui mémorisé dans Tag Ignition
	level = getModuleLogLevel()
	system.util.setLoggingLevel(rootLoggerByesModules, level)	
	
def getLogger(strLog=strDefaultNomLogger):
	"""
		Retourne l'objet logger à partir d'un nom de logger
		
		Args:
			strLog (:obj:'str', optional): Nom du logger
		Returns:
			str: Désignation du logger trouvé. Si logger non trouvé, renvoi celui par défaut
	"""
	global dicoLogger	

	#Initialisation du dictionnaire de reference
	if (len(dicoLogger)==0):
		init()

	if strLog not in ["",None]:
		loggerName = "%s.%s" % (strDefaultNomLogger, strLog)
	else:
		loggerName = "%s" % (strDefaultNomLogger)
		
	"""
	if loggerName not in dicoLogger:		
		myLog = system.util.getLogger(loggerName)		
		dicoLogger[loggerName] = myLog		
		# init niveau de log
		level = getLogLevel()
		system.util.setLoggingLevel(loggerName,level)		
	return dicoLogger[loggerName]
	"""
	
	# v2 sans dico
	myLog = system.util.getLogger(loggerName)
	dicoLogger[loggerName] = myLog
	return myLog
	
	
def formatStrTrace(strWindow,strObjet,strLib,strFct,strMsg,dicoTrace,lstTrace=[],exc_type='',exc_value=''):
	"""
		Fonction commune permettant de formater le message
		
		Args:
			strWindow (str): Nom de la fenêtre appelante
			strObjet (str): Nom de l'objet appelant
			strLib (str): Nom de la librairie
			strFct (str): Nom de la fonction
			strMsg (str): Message à logger
			dicoTrace (dict): Dictionnaire comportant des éléments complémentaires à logger
			lstTrace (:obj:'lst', optional): Liste de la trace java lors d'une exception
			exc_type (:obj:'str', optional): Type d'exception
			exc_value  (:obj:'str', optional): Description de l'exception
		
		Returns:
			str: Chaine de caractère formaté.
	"""
	#Format du message
	strTrace=""
	if strWindow !="":
		strTrace+="Vue: '%s'   "% strWindow
	if strObjet !="":
		strTrace+="Objet: '%s'   "% strObjet
	if strFct !="" and strLib != "":
		strTrace+="Fct: '%s.%s'   "% (strLib,strFct)
	if strFct !="" and strLib == "":
		strTrace+="Fct: '%s'   "% (strFct)			
	if strMsg !="":
		strTrace+="Message: '%s'   "% strMsg
	if 	exc_type!='' or exc_value!='':
		strTrace+="Contexte: Type='%s' Err='%s' Stack="% (exc_type, exc_value)
	for itemTrace in lstTrace:
		strTrace += (str(itemTrace)).replace("\n","   ")
		
	if len(dicoTrace) >0:	
		strDico=""
		for keyTrace, valTrace in dicoTrace.items():
			strDico += "%s='%s'   "%(str(keyTrace),str(valTrace))	
		strTrace += "DicoTrace:{%s}"% strDico
		
	return strTrace
	


def logERROR(strMsg="",evEvent=None,strFct ="",strLib="", dicoTrace={}, strLog=strDefaultNomLogger, session=None):
	"""
		Fonction de log ERROR dans un logger specifique a l'application
		
		Args:
			strMsg (:obj:'str',optional): Descriptif utilisateur
			evEvent (:obj:'obj',optional): Evenement source de l'erreur
			strFct (:obj:'str',optional): Nom de la fonction
			strLib (:obj:'str',optional): Nom de la librairie
			dicoTrace (:obj:'dict',optional): Liste des parametres d'entree de la fonction
			strLog (:obj:'str',optional): Nom du logger a notifier		
	"""
	global strDefaultNomLogger
	import sys
	import traceback
	
	global mdcScriptKey
	global mdcSessionKey
	
	try:

		exc_type, exc_value, exc_traceback = sys.exc_info()
		
		#Recuperation de la pile des erreurs
		lstTrace = traceback.format_exception(exc_type, exc_value,exc_traceback)
		
		strWindow=""
		strObjet=""
		if evEvent !=None :
			#Nom du composant et de ses parent
			strObjet = recursiveFindName(evEvent.source)
			#Nom de la fenetre
			strWindow = system.gui.getParentWindow(evEvent).name
	
		#Format du message
		strTrace=shared.commun.logger.formatStrTrace(strWindow,strObjet,strLib,strFct,strMsg,dicoTrace,lstTrace,exc_type,exc_value)		
		
		#Log
		myLogger = shared.commun.logger.getLogger(strLib)
		myLogger.mdcPut(mdcScriptKey,strLog)		
		if session != None:
			myLogger.mdcPut(mdcSessionKey,session.props.id)		
		myLogger.error(strTrace)
		myLogger.mdcRemove(mdcScriptKey)		

	except:
		pass
	
		
def logINFO(strMsg="",evEvent=None,strFct ="",strLib="", dicoTrace={}, strLog=strDefaultNomLogger, session=None):
	"""			
		Fonction de log INFO dans un logger specifique a l'application
			
		Args:
			strMsg (:obj:'str',optional): Descriptif utilisateur
			evEvent (:obj:'obj',optional): Evenement source de l'erreur
			strFct (:obj:'str',optional): Nom de la fonction
			strLib (:obj:'str',optional): Nom de la librairie
			dicoTrace (:obj:'dict',optional): Liste des parametres d'entree de la fonction
			strLog (:obj:'str',optional): Nom du logger a notifier		
	"""
	
	#import system
	import sys
	import traceback
	
	global mdcScriptKey
	global mdcSessionKey
	
	try:
		myLogger = shared.commun.logger.getLogger(strLib)
		
		if myLogger.isInfoEnabled():	
			strWindow=""
			strObjet=""	
			
			if evEvent !=None:
				#Nom du composant et de ses parent
				strObjet = recursiveFindName(evEvent.source)
				#Nom de la fenetre
				strWindow = system.gui.getParentWindow(evEvent).name
		
			#Format du message
			strTrace=shared.commun.logger.formatStrTrace(strWindow,strObjet,strLib,strFct,strMsg,dicoTrace)
			
			#Log
			myLogger.mdcPut(mdcScriptKey,strLog)
			if session != None:
				myLogger.mdcPut(mdcSessionKey,session.props.id)				
			myLogger.info(strTrace)
			myLogger.mdcRemove(mdcScriptKey)

	except:
		pass
		
		
def logMARK(strLog=strDefaultNomLogger):
	"""			
		Fonction de log d'une marque de niveau INFO dans le logger spécifié
			
		Args:
			strLog (:obj:'str',optional): Nom du logger a notifier		
	"""
	
	#import system
	
	try:

		#Format du message
		strTrace="%s------------------MARK------------------"% (system.date.now())
		
		#Log
		myLogger = shared.commun.logger.getLogger(strLib)
		myLogger.info(strTrace)
	
	except:
		pass

def logDataset(data,strLog=strDefaultNomLogger):
	"""
		Log d'un dataset		
		Return String	
	"""
	try:
		myLogger = shared.commun.logger.getLogger(strLib)
		if data == None:
			strMsg = "dataset = None"
			myLogger.info(strMsg)
		else:
			strMsg = str(data)
			myLogger.info(strMsg)
			strMsg = str(data.getColumnNames())
			myLogger.info(strMsg)
			strMsg = "" 
			for row in range(data.getRowCount()):
				strMsg += ("[%d] " % row)
				for col in range(data.getColumnCount()):
					strMsg += str(data.getValueAt(row,col)) + ' | '
				myLogger.info(strMsg)
				strMsg = ""	
	except:
		pass

def logDEBUG(strMsg="",evEvent=None, strFct ="",strLib="", dicoTrace={}, strLog=strDefaultNomLogger, session=None):
	"""			
		Fonction de log DEBUG dans un logger specifique a l'application
				
		Args:
			strMsg (:obj:'str',optional): Descriptif utilisateur
			evEvent (:obj:'obj',optional): Evenement source de l'erreur
			strFct (:obj:'str',optional): Nom de la fonction
			strLib (:obj:'str',optional): Nom de la librairie
			dicoTrace (:obj:'dict',optional): Liste des parametres d'entree de la fonction
			strLog (:obj:'str',optional): Nom du logger a notifier	
	"""
	#import system
	import sys
	import traceback
	
	global mdcScriptKey
	global mdcSessionKey
	
	try:	
		myLogger = shared.commun.logger.getLogger(strLib)
		
		if myLogger.isDebugEnabled():		
			strWindow=""
			strObjet = ""
				
			if evEvent !=None:
				#Nom du composant et de ses parent
				strObjet = recursiveFindName(evEvent.source)
				#Nom de la fenetre
				strWindow = system.gui.getParentWindow(evEvent).name
				
			#Format message
			strTrace = shared.commun.logger.formatStrTrace(strWindow,strObjet,strLib,strFct,strMsg,dicoTrace)
			#Log
			myLogger.mdcPut(mdcScriptKey,strLog)
			if session != None:
				myLogger.mdcPut(mdcSessionKey,session.props.id)
			myLogger.debug(strTrace)
			myLogger.mdcRemove(mdcScriptKey)	
	except:
		pass


def logTRACE(strMsg="",evEvent=None, strFct ="",strLib="", dicoTrace={}, strLog=strDefaultNomLogger, session=None):
	"""
		Fonction de log TRACE dans un logger specifique a l'application
			
		Args:
			strMsg (:obj:'str',optional): Descriptif utilisateur
			evEvent (:obj:'obj',optional): Evenement source de l'erreur
			strFct (:obj:'str',optional): Nom de la fonction
			strLib (:obj:'str',optional): Nom de la librairie
			dicoTrace (:obj:'dict',optional): Liste des parametres d'entree de la fonction
			strLog (:obj:'str',optional): Nom du logger a notifier	
	"""
	#import system
	import sys
	import traceback
	
	global mdcScriptKey
	global mdcSessionKey
	
	try:	
		myLogger = shared.commun.logger.getLogger(strLib)		
		if myLogger.isTraceEnabled():			
			strWindow=""
			strObjet = ""				
			if evEvent !=None:
				#Nom du composant et de ses parent
				strObjet = recursiveFindName(evEvent.source)
				#Nom de la fenetre
				strWindow = system.gui.getParentWindow(evEvent).name				
			#Format message
			strTrace = shared.commun.logger.formatStrTrace(strWindow,strObjet,strLib,strFct,strMsg,dicoTrace)
			#Log		
			myLogger.mdcPut(mdcScriptKey,strLog)
			if session != None:
				myLogger.mdcPut(mdcSessionKey,session.props.id)			
			myLogger.trace(strTrace)
			myLogger.mdcRemove(mdcScriptKey)	
	except:
		pass

def logWARN(strMsg="",evEvent=None, strFct ="",strLib="", dicoTrace={}, strLog=strDefaultNomLogger, session=None):
	"""
		Fonction de log WARN dans un logger specifique a l'application
		
		Args:
			strMsg (:obj:'str',optional): Descriptif utilisateur
			evEvent (:obj:'obj',optional): Evenement source de l'erreur
			strFct (:obj:'str',optional): Nom de la fonction
			strLib (:obj:'str',optional): Nom de la librairie
			dicoTrace (:obj:'dict',optional): Liste des parametres d'entree de la fonction
			strLog (:obj:'str',optional): Nom du logger a notifier	
	"""
	#import system
	import sys
	import traceback
	
	global mdcScriptKey
	global mdcSessionKey
	
	try:	
		strWindow=""
		strObjet = ""
			
		if evEvent !=None:
			#Nom du composant et de ses parent
			strObjet = recursiveFindName(evEvent.source)
			#Nom de la fenetre
			strWindow = system.gui.getParentWindow(evEvent).name
	
		#Format du message
		strTrace = shared.commun.logger.formatStrTrace(strWindow,strObjet,strLib,strFct,strMsg,dicoTrace)
		#Log
		myLogger = shared.commun.logger.getLogger(strLib)		
		myLogger.mdcPut(mdcScriptKey,strLog)
		if session != None:
			myLogger.mdcPut(mdcSessionKey,session.props.id)
		myLogger.warn(strTrace)
		myLogger.mdcRemove(mdcScriptKey)
	except:
		pass

def recursiveFindName(compSrc):
	"""
		Fonction listant les parents d'un composant
		
		Args:
			compSrc (obj): composant event.source (enfant)
		
		Returns:
			str: String comportant tous les enfants trouvés
	"""
	if compSrc.parent == None or compSrc.name =="Root Container" :
		return str(compSrc.name)
	else:
		return "%s>%s" % (recursiveFindName(compSrc.parent),str(compSrc.name)) 
		

def startLogClient():
	"""
		Fonction initilisation logger dans le scope client
	"""
	global strLib
		
	global logDir
	global logMaxFileSize
	global logMaxIndex
	global logPattern
	
	global appenderName
		
	strFct = "startLogClient"
	
	try:
		init()
		initModules()
		# /!\ Sécurité pour que la fonction ne soit appelée que dans le scope CLIENT/DESIGNER
		scope = getScope()
		# pas dans le scope designer : pour eviter pb demarrage
		if (scope in ["C","D"]):
		
			#https://logback.qos.ch/manual
			#https://logback.qos.ch/apidocs/index.html
			#https://stackoverflow.com/questions/47299109/programatically-add-appender-in-logback-slf4j
			#https://gist.github.com/florianb/10464692
			#https://javaetmoi.com/2018/03/configurez-logback-en-java/
			 
			from org.slf4j import LoggerFactory
			from ch.qos.logback.classic import Logger
			from ch.qos.logback.classic import LoggerContext
			from ch.qos.logback.classic.encoder import PatternLayoutEncoder
			from ch.qos.logback.core import FileAppender
			from ch.qos.logback.core.util import FileSize
			from ch.qos.logback.core.rolling import RollingFileAppender
			from ch.qos.logback.core.rolling import FixedWindowRollingPolicy
			from ch.qos.logback.core.rolling import SizeBasedTriggeringPolicy		
					
			loggerContext = LoggerFactory.getILoggerFactory()
	
			# si le lecteur n'existe pas, on le remplace par c:
			# NB : le répertoire est crée automatiquement si besoin
			if not system.file.fileExists(logDir[:2]):
				logDir = "c:" + logDir[2:]
			
			# pour eviter les conflits qs client et designer sur le même poste
			if scope=="C":
				filename="client-vision"
			else:
				filename="designer"
			
			logPath = "%s/%s.txt" % (logDir,filename) 
			logFileNamePattern = "%s/%s_%s.txt" % (logDir,filename,"%i") 
			
			#print logPath
			#print logFileNamePattern
			
			rfAppender = RollingFileAppender()
			rfAppender.setContext(loggerContext)
			rfAppender.setFile(logPath)
			rfAppender.setName(appenderName)
	
			fwRollingPolicy = FixedWindowRollingPolicy()
			fwRollingPolicy.setContext(loggerContext)		
			fwRollingPolicy.setMinIndex(1)
			fwRollingPolicy.setMaxIndex(logMaxIndex)			
			fwRollingPolicy.setFileNamePattern(logFileNamePattern)
			fwRollingPolicy.setParent(rfAppender)
			fwRollingPolicy.start()
	
			triggeringPolicy = SizeBasedTriggeringPolicy()
			#size = FileSize.valueOf("5 KB")
			#triggeringPolicy.setMaxFileSize(size)
			triggeringPolicy.setMaxFileSize(logMaxFileSize)
			triggeringPolicy.start()
	
			encoder = PatternLayoutEncoder()
			encoder.setContext(loggerContext)
			encoder.setPattern(logPattern)
			encoder.start()
	
			rfAppender.setEncoder(encoder)
			rfAppender.setRollingPolicy(fwRollingPolicy)
			rfAppender.setTriggeringPolicy(triggeringPolicy)
			rfAppender.start()
	
			# Version FileAppender
			#fileAppender = FileAppender()
			#fileAppender.setFile(logFile)
			#fileAppender.setEncoder(patternLayoutEncoder)
			#fileAppender.setContext(loggerContext)
			#fileAppender.start()
			
			# Pour ajouter l'appender à tous les logs du client
			logbackLogger =	LoggerFactory.getLogger(Logger.ROOT_LOGGER_NAME)
	
			# Au cas ou la fonction est appelé plusieur fois
			appender = logbackLogger.getAppender(appenderName)
			
#			if appender != None:
#				appender.stop()
#				logbackLogger.detachAppender(appender)
#				strMsg="End detachAppender"
#				logINFO(strMsg=strMsg,strFct=strFct,strLib=strLib)

# ne recrée pas l'appender suite à delog/relog par exemple
			
			if appender == None:
				logbackLogger.addAppender(rfAppender)
				
			logDEBUG(strMsg=u"End start client log",strFct=strFct,strLib=strLib)
			#/!\ faire les stop dans le shutdown pour fermer correctement les fichiers
			# ou detach suffisant ?			
	except:
		pass

def stopLogClient():
	"""
		Fonction arrêt logger dans le scope client
	"""
	global strLib
	global appenderName
	strFct = "stopLogClient"

	#http://logback.qos.ch/apidocs/ch/qos/logback/classic/Logger.html
		
	try:
		logINFO(strMsg=u"Stop client log",strFct=strFct,strLib=strLib)
		# /!\ Sécurité pour que la fonction ne soit appelée que dans le scope CLIENT/DESIGNER
		scope = getScope()
		if (scope in ["C","D"]):
			from org.slf4j import LoggerFactory
			from ch.qos.logback.classic import Logger			
			logbackLogger =	LoggerFactory.getLogger(Logger.ROOT_LOGGER_NAME)
			appender = logbackLogger.getAppender(appenderName) 
			if appender != None:
				appender.stop()
				#logbackLogger.detachAppender(appenderName)
				logbackLogger.detachAppender(appender)
				strMsg="End detachAppender"
				logINFO(strMsg=strMsg,strFct=strFct,strLib=strLib)
				
			strMsg="End stop client log - appenderName=%s" % appenderName
			logINFO(strMsg=strMsg,strFct=strFct,strLib=strLib)
	except:
		pass
										
def getScope():
	from com.inductiveautomation.ignition.common.model import ApplicationScope 
	try:
		scope = ApplicationScope.getGlobalScope()
		if ApplicationScope.isGateway(scope):
			# Scope Gateway
			return "G"
		elif ApplicationScope.isClient(scope):
			# Scope Client
			return "C"
		elif ApplicationScope.isDesigner(scope):
			# Scope Client
			return "D"
		else:
			return "?"
	except:
		return "?"

			
def dirtyPrint(strMsg):
	"""			
		Fonction de print rapide	
	"""
	global strDefaultNomLogger
	import sys
	import traceback
	
	global mdcScriptKey	
	global mdcSessionKey
	
	try:
		myLogger = shared.commun.logger.getLogger(strDefaultNomLogger)		
		if myLogger.isInfoEnabled():			
			#Log
			myLogger.mdcPut(mdcScriptKey,"")
			if session != None:
				myLogger.mdcPut(mdcSessionKey,session.props.id)
			myLogger.info(strMsg)
			myLogger.mdcRemove(mdcScriptKey)

	except:
		pass
