"""
	Smart Multifaceted Platform ™
	© 2019 - 2022 - Bouygues Energies & Services

	Bibliothèque de gestion video et mur image
"""

import system
import shared.commun.logger
import shared.commun.constantes
import shared.commun.constantesCustom

strLib = "shared.commun.video"

def readCamera_all():
	"""
		readCamera_all
	"""		
	global strLib
	strFct="readCamera_all"	
		
	try:
		dicoConfig = shared.commun.constantes.getAllConstantesVIDEO()
		demoMode = False
		if "demoMode" in dicoConfig:
			demoMode = dicoConfig["demoMode"]
		if demoMode:
			# mode de test avec une ou plusieurs caméra
			demoCamera = dicoConfig["demoCamera"]
			headers = ["id","tag","libelle","IdCam","pathEqt"]
			rows = []
			id = 1			
			for key in demoCamera.keys():				
				tag = key
				libelle = "DEMO CAMERA %s" % key
				idCam = demoCamera[key]["idCam"]
				pathEqt = demoCamera[key]["eqtPath"]
				rows.append([id,tag,libelle,idCam,pathEqt])
				id = id + 1
			data = system.dataset.toDataSet(headers, rows)			
			return data
		else:	
			return None

	except:
		shared.commun.logger.logERROR(strMsg='Erreur runNamedQuery',strFct=strFct,strLib=strLib)
	return None

def rechercherCamera(tagCamera):
	"""
		Fonction recherche données caméra
		Args: tagCamera de la forme 'CDM-EXT-CAF-01'
		Returns dicoCamera 
	"""
	
	import shared.commun.video
	
	global strLib
	strFct="rechercherCamera"	
	
	global tagPathsCamera
	
	dicoCamera = {}
	try:
		dicoConfig = shared.commun.constantes.getAllConstantesVIDEO()
		demoMode = False
		if "demoMode" in dicoConfig:
			demoMode = dicoConfig["demoMode"]
		if demoMode:
			# mode de test avec une ou plusieurs caméra
			demoCamera = dicoConfig["demoCamera"]			
			if tagCamera in demoCamera:
				key = tagCamera
			else:
				# première caméra de la liste, pour mode demo
				key = demoCamera[demoCamera.keys()[0]]
									
			dicoCamera["endpointAddress"] = demoCamera[key]["endpointAddress"]
			dicoCamera["endpointPassword"] = demoCamera[key]["endpointPassword"]
			dicoCamera["endpointUser"] = demoCamera[key]["endpointUser"]
			
			dicoCamera["streamPassword"] = demoCamera[key]["streamPassword"]
			dicoCamera["streamUser"] = demoCamera[key]["streamUser"]
			
			dicoCamera["ptzActivePanTilt"] = demoCamera[key]["ptzActivePanTilt"]
			dicoCamera["ptzActiveZoom"] = demoCamera[key]["ptzActiveZoom"]
			dicoCamera["ptzProfileToken"] = demoCamera[key]["ptzProfileToken"]
			
			dicoCamera["streamUrl"] = demoCamera[key]["streamUrl"]
			
			dicoCamera["idCam"] = demoCamera[key]["idCam"]
			dicoCamera["eqtPath"] = demoCamera[key]["eqtPath"]
			
			dicoCamera["tag"] = key
	except:
		shared.commun.logger.logERROR(strMsg='Erreur runNamedQuery',strFct=strFct,strLib=strLib)
	return dicoCamera

def listerTestPattern():
	global strLib
	strFct="listerTestPattern"
	
	try:	
		patterns = []	
		patterns.append("pattern=smpte")
		patterns.append("pattern=snow")
		patterns.append("pattern=black")
		patterns.append("pattern=white")
		patterns.append("pattern=red")
		patterns.append("pattern=green")
		patterns.append("pattern=blue")
		patterns.append("pattern=checkers-1")
		patterns.append("pattern=checkers-2")
		patterns.append("pattern=checkers-4")
		patterns.append("pattern=checkers-8")
		patterns.append("pattern=circular")
		patterns.append("pattern=blink")
		patterns.append("pattern=smpte75")
		patterns.append("pattern=zone-plate")
		patterns.append("pattern=gamut")
		patterns.append("pattern=chroma-zone-plate")
		patterns.append("pattern=solid-color")
		patterns.append("pattern=ball")
		patterns.append("pattern=smpte100")
		patterns.append("pattern=bar")
		patterns.append("pattern=pinwheel")
		patterns.append("pattern=spokes")
		patterns.append("pattern=gradient")
		patterns.append("pattern=colors")
		
		return patterns 	
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)

def cleanPTZReservation():
	"""
		En version prespective, le popup vidéo pages/STD/VID/STD_VID_01
		réserve une connexion pour le pilotage PTZ parmi celle disponible.
		La clé de réservation est le pageId
		Sur fermeture du popup ou de la page (onShutdown de templates/STD/Video/PTZ_MIRE)
		effectue la libération
		En cas de fermeture brutal du navigateur, la session n'est pas libérée
		Cette fonctionn permet de vérifier que les réservations PTZ correspondent à des pages encore actives
		sinon on effectue la libération automatique pour que d'autres clients puissent se connecter. 
	"""
	global strLib
	strFct="cleanPTZReservation"	
	try:
		import system.byes.onvif.multiple
		projectFilter = shared.commun.constantesCustom.PROJECT
		pageIds = []		
		sessions = system.perspective.getSessionInfo(projectFilter=projectFilter)
		for session in sessions:
			pageIds = pageIds + list(session["pageIds"])
	
		keys = system.byes.onvif.multiple.ptzGetPortsKeys()
		
		for key in keys:
			if key not in pageIds:
				strMsg = u"No active session found for key(=pageId) : %s => ptzReleasePort" % key
				shared.commun.logger.logINFO(strMsg=strMsg,strFct=strFct,strLib=strLib)		
				system.byes.onvif.multiple.ptzReleasePort(key)
	except:
		shared.commun.logger.logERROR(strFct=strFct,strLib=strLib)
